@@ -1,16 +1,18 @@
-import hashlib
+import arrow
 from copy import deepcopy
+import hashlib
+import jinja2
 import json
 import logging
 import os
 import pprint
-
-import arrow
-import jinja2
+import re
 import yaml
 
-from beetmoverscript.constants import (HASH_BLOCK_SIZE, STAGE_PLATFORM_MAP,
-                                       TEMPLATE_KEY_PLATFORMS, RELEASE_ACTIONS)
+from beetmoverscript.constants import (
+    HASH_BLOCK_SIZE, TEMPLATE_KEY_PLATFORMS,
+    RELEASE_ACTIONS, PROMOTION_ACTIONS, PRODUCT_TO_PATH
+)
 
 log = logging.getLogger(__name__)
 
@@ -51,16 +53,32 @@ def write_file(path, contents):
         fh.write(contents)
 
 
-def is_action_a_release_shipping(action):
-    """Function to return boolean if we're shipping a release as opposed to a
+def is_release_action(action):
+    """Function to return boolean if we're publishing a release as opposed to a
     nightly release or something else. Does that by checking the action type.
-    Currently the only action that supports release-releases is
-    `push-to-candidates` for which we need to enforce the `version` and
-    `build_number` grabbing.
     """
     return action in RELEASE_ACTIONS
 
 
+def is_promotion_action(action):
+    """Function to return boolean if we're promoting a release as opposed to a
+    nightly or something else. Does that by checking the action type.
+    """
+    return action in PROMOTION_ACTIONS
+
+
+def get_product_name(appName, tmpl_key):
+    if tmpl_key == "devedition":
+        # XXX: this check is helps reuse this function in both
+        # returning the proper templates file but also for the release name in
+        # Balrog manifest that beetmover is uploading upon successful run
+        if appName[0].isupper():
+            return "Devedition"
+        else:
+            return "devedition"
+    return appName
+
+
 def generate_beetmover_template_args(context):
     task = context.task
     release_props = context.release_props
@@ -75,9 +93,11 @@ def generate_beetmover_template_args(context):
         "product": release_props["appName"],
         "stage_platform": release_props["stage_platform"],
         "platform": release_props["platform"],
+        "buildid": release_props["buildid"],
+        "partials": get_partials_props(task),
     }
 
-    if is_action_a_release_shipping(context.action):
+    if is_promotion_action(context.action) or is_release_action(context.action):
         tmpl_args["build_number"] = task['payload']['build_number']
         tmpl_args["version"] = task['payload']['version']
 
@@ -86,7 +106,8 @@ def generate_beetmover_template_args(context):
 
     if 'locale' in task["payload"]:
         tmpl_args["locale"] = task["payload"]["locale"]
-        tmpl_args["template_key"] = "%s_%s_repacks" % (release_props["appName"].lower(), tmpl_bucket)
+        product_name = get_product_name(release_props["appName"].lower(), tmpl_key_platform)
+        tmpl_args["template_key"] = "%s_%s_repacks" % (product_name, tmpl_bucket)
     else:
         tmpl_args["template_key"] = "%s_%s" % (tmpl_key_platform, tmpl_bucket)
 
@@ -116,28 +137,11 @@ def generate_beetmover_manifest(context):
     return manifest
 
 
-def update_props(props, platform_mapping):
-    """Function to alter the `stage_platform` field from balrog_props to their
-    corresponding correct values for certain platforms. Please note that for
-    l10n jobs the `stage_platform` field is in fact called `platform` hence
-    the defaulting below."""
-    props = deepcopy(props)
-    # en-US jobs have the platform set in the `stage_platform` field while
-    # l10n jobs have it set under `platform`. This is merely an uniformization
-    # under the `stage_platform` field that is needed later on in the templates
-    stage_platform = props.get("stage_platform", props.get("platform"))
-    props["stage_platform"] = stage_platform
-    # for some products/platforms this mapping is not needed, hence the default
-    props["platform"] = platform_mapping.get(stage_platform,
-                                             stage_platform)
-    return props
-
-
-def get_release_props(initial_release_props_file, platform_mapping=STAGE_PLATFORM_MAP):
-    """determined via parsing the Nightly build job's balrog_props.json and
-    expanded the properties with props beetmover knows about."""
-    props = load_json(initial_release_props_file)['properties']
-    return update_props(props, platform_mapping)
+def get_partials_props(task):
+    """Examine contents of task.json (stored in context.task) and extract
+    partials mapping data from the 'extra' field"""
+    partials = task.get('extra', {}).get('partials', {})
+    return {p['artifact_name']: p for p in partials}
 
 
 def alter_unpretty_contents(context, blobs, mappings):
@@ -164,3 +168,28 @@ def alter_unpretty_contents(context, blobs, mappings):
 
             if pretty_contents != contents:
                 write_json(source, pretty_contents)
+
+
+def get_candidates_prefix(product, version, build_number):
+    return "{}candidates/{}-candidates/build{}/".format(
+        PRODUCT_TO_PATH[product], version, str(build_number)
+    )
+
+
+def get_releases_prefix(product, version):
+    return "{}releases/{}/".format(PRODUCT_TO_PATH[product], version)
+
+
+def matches_exclude(keyname, excludes):
+    for exclude in excludes:
+        if re.search(exclude, keyname):
+            return True
+    return False
+
+
+def get_creds(context):
+    return context.config['bucket_config'][context.bucket]['credentials']
+
+
+def get_bucket_name(context, product):
+    return context.config['bucket_config'][context.bucket]['buckets'][product]
